/*
 *   imon.h:	LIRC/VFD/LCD driver for SoundGraph iMON IR/VFD/LCD
 *		including the iMON PAD model
 *
 *   Copyright(C) 2009  Jarod Wilson <jarod@wilsonet.com>
 *   Portions based on the original lirc_imon driver,
 *	Copyright(C) 2004  Venky Raju(dev@venky.ws)
 *
 *   imon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

static const struct key_entry imon_remote_key_table[] = {
	/* keys sorted mostly by frequency of use to optimize lookups */
	{ KE_KEY, 0x2a8195b7, { KEY_REWIND } },
	{ KE_KEY, 0x298315b7, { KEY_REWIND } },
	{ KE_KEY, 0x2b8115b7, { KEY_FASTFORWARD } },
	{ KE_KEY, 0x2b8315b7, { KEY_FASTFORWARD } },
	{ KE_KEY, 0x2b9115b7, { KEY_PREVIOUS } },
	{ KE_KEY, 0x298195b7, { KEY_NEXT } },

	{ KE_KEY, 0x2a8115b7, { KEY_PLAY } },
	{ KE_KEY, 0x2a8315b7, { KEY_PLAY } },
	{ KE_KEY, 0x2a9115b7, { KEY_PAUSE } },
	{ KE_KEY, 0x2b9715b7, { KEY_STOP } },
	{ KE_KEY, 0x298115b7, { KEY_RECORD } },

	{ KE_KEY, 0x01008000, { KEY_UP } },
	{ KE_KEY, 0x01007f00, { KEY_DOWN } },
	{ KE_KEY, 0x01000080, { KEY_LEFT } },
	{ KE_KEY, 0x0100007f, { KEY_RIGHT } },

	{ KE_KEY, 0x2aa515b7, { KEY_UP } },
	{ KE_KEY, 0x289515b7, { KEY_DOWN } },
	{ KE_KEY, 0x29a515b7, { KEY_LEFT } },
	{ KE_KEY, 0x2ba515b7, { KEY_RIGHT } },

	{ KE_KEY, 0x0200002c, { KEY_SPACE } }, /* Select/Space */
	{ KE_KEY, 0x02000028, { KEY_ENTER } },
	{ KE_KEY, 0x288195b7, { KEY_EXIT } },
	{ KE_KEY, 0x02000029, { KEY_ESC } },
	{ KE_KEY, 0x0200002a, { KEY_BACKSPACE } },

	{ KE_KEY, 0x2b9595b7, { KEY_MUTE } },
	{ KE_KEY, 0x28a395b7, { KEY_VOLUMEUP } },
	{ KE_KEY, 0x28a595b7, { KEY_VOLUMEDOWN } },
	{ KE_KEY, 0x289395b7, { KEY_CHANNELUP } },
	{ KE_KEY, 0x288795b7, { KEY_CHANNELDOWN } },

	{ KE_KEY, 0x0200001e, { KEY_NUMERIC_1 } },
	{ KE_KEY, 0x0200001f, { KEY_NUMERIC_2 } },
	{ KE_KEY, 0x02000020, { KEY_NUMERIC_3 } },
	{ KE_KEY, 0x02000021, { KEY_NUMERIC_4 } },
	{ KE_KEY, 0x02000022, { KEY_NUMERIC_5 } },
	{ KE_KEY, 0x02000023, { KEY_NUMERIC_6 } },
	{ KE_KEY, 0x02000024, { KEY_NUMERIC_7 } },
	{ KE_KEY, 0x02000025, { KEY_NUMERIC_8 } },
	{ KE_KEY, 0x02000026, { KEY_NUMERIC_9 } },
	{ KE_KEY, 0x02000027, { KEY_NUMERIC_0 } },

	{ KE_KEY, 0x02200025, { KEY_NUMERIC_STAR } },
	{ KE_KEY, 0x02200020, { KEY_NUMERIC_POUND } },

	{ KE_KEY, 0x2b8515b7, { KEY_VIDEO } },
	{ KE_KEY, 0x299195b7, { KEY_AUDIO } },
	{ KE_KEY, 0x2ba115b7, { KEY_CAMERA } },
	{ KE_KEY, 0x28a515b7, { KEY_TV } },
	{ KE_KEY, 0x29a395b7, { KEY_DVD } },
	{ KE_KEY, 0x29a295b7, { KEY_DVD } },

	/* the Menu key between DVD and Subtitle on the RM-200... */
	{ KE_KEY, 0x2ba385b7, { KEY_MENU } },
	{ KE_KEY, 0x2ba395b7, { KEY_MENU } },

	{ KE_KEY, 0x288515b7, { KEY_BOOKMARKS } },
	{ KE_KEY, 0x2ab715b7, { KEY_MEDIA } }, /* Thumbnail */
	{ KE_KEY, 0x298595b7, { KEY_SUBTITLE } },
	{ KE_KEY, 0x2b8595b7, { KEY_LANGUAGE } },

	{ KE_KEY, 0x29a595b7, { KEY_ZOOM } },
	{ KE_KEY, 0x2aa395b7, { KEY_SCREEN } }, /* FullScreen */

	{ KE_KEY, 0x299115b7, { KEY_KEYBOARD } },
	{ KE_KEY, 0x299135b7, { KEY_KEYBOARD } },

	{ KE_KEY, 0x01010000, { BTN_LEFT } },
	{ KE_KEY, 0x01020000, { BTN_RIGHT } },
	{ KE_KEY, 0x01010080, { BTN_LEFT } },
	{ KE_KEY, 0x01020080, { BTN_RIGHT } },

	{ KE_KEY, 0x2a9395b7, { KEY_CYCLEWINDOWS } }, /* TaskSwitcher */
	{ KE_KEY, 0x2b8395b7, { KEY_TIME } }, /* Timer */

	{ KE_KEY, 0x289115b7, { KEY_POWER } },
	{ KE_KEY, 0x29b195b7, { KEY_EJECTCD } }, /* the one next to play */
	{ KE_KEY, 0x299395b7, { KEY_EJECTCLOSECD } }, /* eject (above TaskSwitcher) */

	{ KE_KEY, 0x02800000, { KEY_MENU } }, /* Left Menu */
	{ KE_KEY, 0x02000065, { KEY_COMPOSE } }, /* RightMenu */
	{ KE_KEY, 0x2ab195b7, { KEY_PROG1 } }, /* Go */
	{ KE_KEY, 0x29b715b7, { KEY_DASHBOARD } }, /* AppLauncher */
	{ KE_END, 0 }
};

static const struct key_entry imon_mce_key_table[] = {
	/* keys sorted mostly by frequency of use to optimize lookups */
	{ KE_KEY, 0x800f8415, { KEY_REWIND } },
	{ KE_KEY, 0x800f8414, { KEY_FASTFORWARD } },
	{ KE_KEY, 0x800f841b, { KEY_PREVIOUS } },
	{ KE_KEY, 0x800f841a, { KEY_NEXT } },

	{ KE_KEY, 0x800f8416, { KEY_PLAY } },
	{ KE_KEY, 0x800f8418, { KEY_PAUSE } },
	{ KE_KEY, 0x800f8418, { KEY_PAUSE } },
	{ KE_KEY, 0x800f8419, { KEY_STOP } },
	{ KE_KEY, 0x800f8417, { KEY_RECORD } },

	{ KE_KEY, 0x02000052, { KEY_UP } },
	{ KE_KEY, 0x02000051, { KEY_DOWN } },
	{ KE_KEY, 0x02000050, { KEY_LEFT } },
	{ KE_KEY, 0x0200004f, { KEY_RIGHT } },

	{ KE_KEY, 0x02000028, { KEY_ENTER } },
/* the OK and Enter buttons decode to the same value
	{ KE_KEY, 0x02000028, { KEY_OK } }, */
	{ KE_KEY, 0x0200002a, { KEY_EXIT } },
	{ KE_KEY, 0x02000029, { KEY_DELETE } },

	{ KE_KEY, 0x800f840e, { KEY_MUTE } },
	{ KE_KEY, 0x800f8410, { KEY_VOLUMEUP } },
	{ KE_KEY, 0x800f8411, { KEY_VOLUMEDOWN } },
	{ KE_KEY, 0x800f8412, { KEY_CHANNELUP } },
	{ KE_KEY, 0x800f8413, { KEY_CHANNELDOWN } },

	{ KE_KEY, 0x0200001e, { KEY_NUMERIC_1 } },
	{ KE_KEY, 0x0200001f, { KEY_NUMERIC_2 } },
	{ KE_KEY, 0x02000020, { KEY_NUMERIC_3 } },
	{ KE_KEY, 0x02000021, { KEY_NUMERIC_4 } },
	{ KE_KEY, 0x02000022, { KEY_NUMERIC_5 } },
	{ KE_KEY, 0x02000023, { KEY_NUMERIC_6 } },
	{ KE_KEY, 0x02000024, { KEY_NUMERIC_7 } },
	{ KE_KEY, 0x02000025, { KEY_NUMERIC_8 } },
	{ KE_KEY, 0x02000026, { KEY_NUMERIC_9 } },
	{ KE_KEY, 0x02000027, { KEY_NUMERIC_0 } },

	{ KE_KEY, 0x02200025, { KEY_NUMERIC_STAR } },
	{ KE_KEY, 0x02200020, { KEY_NUMERIC_POUND } },

	{ KE_KEY, 0x800f8446, { KEY_TV } },
	{ KE_KEY, 0x800f8447, { KEY_AUDIO } },
	{ KE_KEY, 0x800f8448, { KEY_PVR } }, /* RecordedTV */
	{ KE_KEY, 0x800f8449, { KEY_CAMERA } },
	{ KE_KEY, 0x800f844a, { KEY_VIDEO } },
	{ KE_KEY, 0x800f8424, { KEY_DVD } },
	{ KE_KEY, 0x800f8425, { KEY_TUNER } }, /* LiveTV */

	{ KE_KEY, 0x800f845b, { KEY_RED } },
	{ KE_KEY, 0x800f845c, { KEY_GREEN } },
	{ KE_KEY, 0x800f845d, { KEY_YELLOW } },
	{ KE_KEY, 0x800f845e, { KEY_BLUE } },

	{ KE_KEY, 0x800f840f, { KEY_INFO } },
	{ KE_KEY, 0x800f8426, { KEY_EPG } }, /* Guide */
	{ KE_KEY, 0x800f845a, { KEY_SUBTITLE } }, /* Caption */

	{ KE_KEY, 0x800f840c, { KEY_POWER } },
	{ KE_KEY, 0x800f840d, { KEY_PROG1 } }, /* Windows MCE button */
	{ KE_END, 0 }

};

static const struct {
	u64 hw_code;
	u16 keycode;
} imon_panel_key_table[] = {
	{ 0x000000000f000fee, KEY_PROG1 }, /* Go */
	{ 0x000000001f000fee, KEY_AUDIO },
	{ 0x0000000020000fee, KEY_VIDEO },
	{ 0x0000000021000fee, KEY_CAMERA },
	{ 0x0000000027000fee, KEY_DVD },
/* the TV key on my panel is broken, doesn't work under any OS
	{ 0x0000000000000fee, KEY_TV }, */
	{ 0x0000000005000fee, KEY_PREVIOUS },
	{ 0x0000000007000fee, KEY_REWIND },
	{ 0x0000000004000fee, KEY_STOP },
	{ 0x000000003c000fee, KEY_PLAYPAUSE },
	{ 0x0000000008000fee, KEY_FASTFORWARD },
	{ 0x0000000006000fee, KEY_NEXT },
	{ 0x0000000100000fee, KEY_RIGHT },
	{ 0x0000010000000fee, KEY_LEFT },
	{ 0x000000003d000fee, KEY_SELECT },
	{ 0x0001000000000fee, KEY_VOLUMEUP },
	{ 0x0100000000000fee, KEY_VOLUMEDOWN },
	{ 0x0000000001000fee, KEY_MUTE },
};
